@@ -1498,7 +1498,7 @@ yyreturnlab:
 
 const char *old_to_new_ctrl_flag(const char *old)
 {
-    static const char *clist[] = {
+    static const char *const clist[] = {
 	"requisite",
 	"required",
 	"sufficient",
