@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.7.6.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
@@ -46,10 +46,10 @@
    USER NAME SPACE" below.  */
 
 /* Identify Bison output, and Bison version.  */
-#define YYBISON 30706
+#define YYBISON 30802
 
 /* Bison version string.  */
-#define YYBISON_VERSION "3.7.6"
+#define YYBISON_VERSION "3.8.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -179,8 +179,10 @@ typedef union YYSTYPE YYSTYPE;
 
 extern YYSTYPE yylval;
 
+
 int yyparse (void);
 
+
 #endif /* !YY_YY_PARSE_Y_H_INCLUDED  */
 /* Symbol kind.  */
 enum yysymbol_kind_t
@@ -357,12 +359,18 @@ typedef int yy_state_fast_t;
 # define YY_USE(E) /* empty */
 #endif
 
-#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
 /* Suppress an incorrect diagnostic about yylval being uninitialized.  */
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                            \
+#if defined __GNUC__ && ! defined __ICC && 406 <= __GNUC__ * 100 + __GNUC_MINOR__
+# if __GNUC__ * 100 + __GNUC_MINOR__ < 407
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
+    _Pragma ("GCC diagnostic push")                                     \
+    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")
+# else
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
     _Pragma ("GCC diagnostic push")                                     \
     _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
     _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
+# endif
 # define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
     _Pragma ("GCC diagnostic pop")
 #else
@@ -577,7 +585,7 @@ static const yytype_int8 yytranslate[] =
 };
 
 #if YYDEBUG
-  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
+/* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
        0,    39,    39,    40,    44,    53,    72,    99,   128,   131,
@@ -609,15 +617,6 @@ yysymbol_name (yysymbol_kind_t yysymbol)
 }
 #endif
 
-#ifdef YYPRINT
-/* YYTOKNUM[NUM] -- (External) token number corresponding to the
-   (internal) symbol number NUM (which must be that of a token).  */
-static const yytype_int16 yytoknum[] =
-{
-       0,   256,   257,   258,   259,   260,   261,   262,   263,   264
-};
-#endif
-
 #define YYPACT_NINF (-3)
 
 #define yypact_value_is_default(Yyn) \
@@ -628,38 +627,38 @@ static const yytype_int16 yytoknum[] =
 #define yytable_value_is_error(Yyn) \
   0
 
-  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
-     STATE-NUM.  */
+/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
+   STATE-NUM.  */
 static const yytype_int8 yypact[] =
 {
       -3,     0,    -3,    -3,     5,    -3,    -3,    -3,    -3,    -3,
       -3,    -3,    17,    12,    -3,    -3,    -3,    -2,    -3
 };
 
-  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
-     Performed when YYTABLE does not specify something else to do.  Zero
-     means the default is an error.  */
+/* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
+   Performed when YYTABLE does not specify something else to do.  Zero
+   means the default is an error.  */
 static const yytype_int8 yydefact[] =
 {
        2,     8,     1,     3,     0,    15,    14,    13,    10,     4,
       12,     8,     9,     0,    11,     5,     8,     0,     6
 };
 
-  /* YYPGOTO[NTERM-NUM].  */
+/* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
       -3,    -3,    11,    -3
 };
 
-  /* YYDEFGOTO[NTERM-NUM].  */
+/* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
        0,     1,     4,    12
 };
 
-  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
-     positive, shift that token.  If negative, reduce the rule whose
-     number is the opposite.  If YYTABLE_NINF, syntax error.  */
+/* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
+   positive, shift that token.  If negative, reduce the rule whose
+   number is the opposite.  If YYTABLE_NINF, syntax error.  */
 static const yytype_int8 yytable[] =
 {
        2,     5,     6,     7,     8,    18,    10,     3,     5,     6,
@@ -674,22 +673,22 @@ static const yytype_int8 yycheck[] =
        8,     9,    11,     6,    -1,    -1,    -1,    16
 };
 
-  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
-     symbol of state STATE-NUM.  */
+/* YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
+   state STATE-NUM.  */
 static const yytype_int8 yystos[] =
 {
        0,    11,     0,     7,    12,     3,     4,     5,     6,     7,
        8,     9,    13,    12,     6,     7,     9,    12,     7
 };
 
-  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
+/* YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr1[] =
 {
        0,    10,    11,    11,    11,    11,    11,    11,    12,    12,
       13,    13,    13,    13,    13,    13
 };
 
-  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
+/* YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr2[] =
 {
        0,     2,     0,     2,     3,     5,     7,     7,     0,     2,
@@ -705,6 +704,7 @@ enum { YYENOMEM = -2 };
 #define YYACCEPT        goto yyacceptlab
 #define YYABORT         goto yyabortlab
 #define YYERROR         goto yyerrorlab
+#define YYNOMEM         goto yyexhaustedlab
 
 
 #define YYRECOVERING()  (!!yyerrstatus)
@@ -745,10 +745,7 @@ do {                                            \
     YYFPRINTF Args;                             \
 } while (0)
 
-/* This macro is provided for backward compatibility. */
-# ifndef YY_LOCATION_PRINT
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
+
 
 
 # define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
@@ -775,10 +772,6 @@ yy_symbol_value_print (FILE *yyo,
   YY_USE (yyoutput);
   if (!yyvaluep)
     return;
-# ifdef YYPRINT
-  if (yykind < YYNTOKENS)
-    YYPRINT (yyo, yytoknum[yykind], *yyvaluep);
-# endif
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   YY_USE (yykind);
   YY_IGNORE_MAYBE_UNINITIALIZED_END
@@ -963,6 +956,7 @@ yyparse (void)
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yychar = YYEMPTY; /* Cause a token to be read.  */
+
   goto yysetstate;
 
 
@@ -988,7 +982,7 @@ yysetstate:
 
   if (yyss + yystacksize - 1 <= yyssp)
 #if !defined yyoverflow && !defined YYSTACK_RELOCATE
-    goto yyexhaustedlab;
+    YYNOMEM;
 #else
     {
       /* Get the current used size of the three stacks, in elements.  */
@@ -1016,7 +1010,7 @@ yysetstate:
 # else /* defined YYSTACK_RELOCATE */
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
-        goto yyexhaustedlab;
+        YYNOMEM;
       yystacksize *= 2;
       if (YYMAXDEPTH < yystacksize)
         yystacksize = YYMAXDEPTH;
@@ -1027,7 +1021,7 @@ yysetstate:
           YY_CAST (union yyalloc *,
                    YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
         if (! yyptr)
-          goto yyexhaustedlab;
+          YYNOMEM;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
         YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
@@ -1049,6 +1043,7 @@ yysetstate:
     }
 #endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */
 
+
   if (yystate == YYFINAL)
     YYACCEPT;
 
@@ -1166,7 +1161,7 @@ yyreduce:
     printf("\n");
     ++line;
 }
-#line 1170 "parse_y.c"
+#line 1165 "parse_y.c"
     break;
 
   case 4: /* doc: doc stuff NEWLINE  */
@@ -1180,7 +1175,7 @@ yyreduce:
     indent = 1;
     ++line;
 }
-#line 1184 "parse_y.c"
+#line 1179 "parse_y.c"
     break;
 
   case 5: /* doc: doc stuff RIGHT stuff NEWLINE  */
@@ -1204,7 +1199,7 @@ yyreduce:
     indent = 1;
     ++line;
 }
-#line 1208 "parse_y.c"
+#line 1203 "parse_y.c"
     break;
 
   case 6: /* doc: doc stuff RIGHT stuff RIGHT stuff NEWLINE  */
@@ -1236,7 +1231,7 @@ yyreduce:
     indent = 1;
     ++line;
 }
-#line 1240 "parse_y.c"
+#line 1235 "parse_y.c"
     break;
 
   case 7: /* doc: doc stuff RIGHT stuff RIGHT stuff NEWLINE  */
@@ -1268,7 +1263,7 @@ yyreduce:
     indent = 1;
     ++line;
 }
-#line 1272 "parse_y.c"
+#line 1267 "parse_y.c"
     break;
 
   case 8: /* stuff: %empty  */
@@ -1276,7 +1271,7 @@ yyreduce:
        {
     (yyval.string) = strdup("");
 }
-#line 1280 "parse_y.c"
+#line 1275 "parse_y.c"
     break;
 
   case 9: /* stuff: stuff text  */
@@ -1287,7 +1282,7 @@ yyreduce:
     free((yyvsp[-1].string));
     free((yyvsp[0].string));
 }
-#line 1291 "parse_y.c"
+#line 1286 "parse_y.c"
     break;
 
   case 10: /* text: CHAR  */
@@ -1295,7 +1290,7 @@ yyreduce:
            {
     (yyval.string) = strdup(yytext);
 }
-#line 1299 "parse_y.c"
+#line 1294 "parse_y.c"
     break;
 
   case 11: /* text: text CHAR  */
@@ -1305,7 +1300,7 @@ yyreduce:
     sprintf((yyval.string),"%s%s", (yyvsp[-1].string), yytext);
     free((yyvsp[-1].string));
 }
-#line 1309 "parse_y.c"
+#line 1304 "parse_y.c"
     break;
 
   case 12: /* text: NO_INDENT  */
@@ -1314,7 +1309,7 @@ yyreduce:
     (yyval.string) = strdup("");
     indent = 0;
 }
-#line 1318 "parse_y.c"
+#line 1313 "parse_y.c"
     break;
 
   case 13: /* text: HASH  */
@@ -1322,7 +1317,7 @@ yyreduce:
        {
     (yyval.string) = strdup("#");
 }
-#line 1326 "parse_y.c"
+#line 1321 "parse_y.c"
     break;
 
   case 14: /* text: LABEL  */
@@ -1333,7 +1328,7 @@ yyreduce:
 	(yyval.string) = strdup("");
     }
 }
-#line 1337 "parse_y.c"
+#line 1332 "parse_y.c"
     break;
 
   case 15: /* text: NEW_COUNTER  */
@@ -1341,11 +1336,11 @@ yyreduce:
               {
     (yyval.string) = new_counter(yytext);
 }
-#line 1345 "parse_y.c"
+#line 1340 "parse_y.c"
     break;
 
 
-#line 1349 "parse_y.c"
+#line 1344 "parse_y.c"
 
       default: break;
     }
@@ -1427,6 +1422,7 @@ yyerrorlab:
      label yyerrorlab therefore never appears in user code.  */
   if (0)
     YYERROR;
+  ++yynerrs;
 
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
@@ -1487,7 +1483,7 @@ yyerrlab1:
 `-------------------------------------*/
 yyacceptlab:
   yyresult = 0;
-  goto yyreturn;
+  goto yyreturnlab;
 
 
 /*-----------------------------------.
@@ -1495,24 +1491,22 @@ yyacceptlab:
 `-----------------------------------*/
 yyabortlab:
   yyresult = 1;
-  goto yyreturn;
+  goto yyreturnlab;
 
 
-#if !defined yyoverflow
-/*-------------------------------------------------.
-| yyexhaustedlab -- memory exhaustion comes here.  |
-`-------------------------------------------------*/
+/*-----------------------------------------------------------.
+| yyexhaustedlab -- YYNOMEM (memory exhaustion) comes here.  |
+`-----------------------------------------------------------*/
 yyexhaustedlab:
   yyerror (YY_("memory exhausted"));
   yyresult = 2;
-  goto yyreturn;
-#endif
+  goto yyreturnlab;
 
 
-/*-------------------------------------------------------.
-| yyreturn -- parsing is finished, clean up and return.  |
-`-------------------------------------------------------*/
-yyreturn:
+/*----------------------------------------------------------.
+| yyreturnlab -- parsing is finished, clean up and return.  |
+`----------------------------------------------------------*/
+yyreturnlab:
   if (yychar != YYEMPTY)
     {
       /* Make sure we have latest lookahead translation.  See comments at
@@ -1660,9 +1654,7 @@ char *new_counter(const char *key)
 
     counter_root = set_key(counter_root, key, new);
 
-    if (last_label) {
-	free(last_label);
-    }
+    free(last_label);
     last_label = strdup(new);
 
     return new;
