@@ -62,13 +62,13 @@
 #define LIMITS_DEF_KERNEL   5 /* limit was set from /proc/1/limits */
 #define LIMITS_DEF_NONE     6 /* this limit was not set yet */
 
-#define LIMIT_RANGE_ERR    -1 /* error in specified uid/gid range */
+#define LIMIT_RANGE_ERR  (-1) /* error in specified uid/gid range */
 #define LIMIT_RANGE_NONE    0 /* no range specified */
 #define LIMIT_RANGE_ONE     1 /* exact uid/gid specified (:max_uid)*/
 #define LIMIT_RANGE_MIN     2 /* only minimum uid/gid specified (min_uid:) */
 #define LIMIT_RANGE_MM      3 /* both min and max uid/gid specified (min_uid:max_uid) */
 
-static const char *limits_def_names[] = {
+static const char *const limits_def_names[] = {
        "USER",
        "GROUP",
        "ALLGROUP",
@@ -100,11 +100,11 @@ struct pam_limit_s {
     char *login_group;
 };
 
-#define LIMIT_LOGIN RLIM_NLIMITS+1
-#define LIMIT_NUMSYSLOGINS RLIM_NLIMITS+2
+#define LIMIT_LOGIN (RLIM_NLIMITS+1)
+#define LIMIT_NUMSYSLOGINS (RLIM_NLIMITS+2)
 
-#define LIMIT_PRI RLIM_NLIMITS+3
-#define LIMIT_NONEWPRIVS RLIM_NLIMITS+4
+#define LIMIT_PRI (RLIM_NLIMITS+3)
+#define LIMIT_NONEWPRIVS (RLIM_NLIMITS+4)
 
 #define LIMIT_SOFT  1
 #define LIMIT_HARD  2
@@ -323,7 +323,7 @@ check_logins (pam_handle_t *pamh, const char *name, int limit, int ctrl,
 }
 
 #ifdef __linux__
-static const char *lnames[RLIM_NLIMITS] = {
+static const char *const lnames[RLIM_NLIMITS] = {
         [RLIMIT_CPU] = "Max cpu time",
         [RLIMIT_FSIZE] = "Max file size",
         [RLIMIT_DATA] = "Max data size",
@@ -389,7 +389,7 @@ static rlim_t str2rlim_t(char *value) {
         pos--; \
         while (pos && line[pos] != ' ') pos--; \
         if (!pos) continue; \
-        item = line + pos + 1; \
+        (item) = line + pos + 1; \
 }
 
 static void parse_kernel_limits(pam_handle_t *pamh, struct pam_limit_s *pl, int ctrl)
@@ -771,7 +771,6 @@ process_limit (const pam_handle_t *pamh, int source, const char *lim_type,
 	    }
 	}
     }
-    return;
 }
 
 static int
@@ -832,6 +831,26 @@ set_if_null(char **dest, char *def)
     return 1;
 }
 
+static char *
+trim(char *s)
+{
+    char *p;
+
+    if (s == NULL)
+	return NULL;
+
+    while (*s == ' ' || *s == '\t')
+	s++;
+
+    if (*s == '\0')
+	return NULL;
+
+    p = s + strlen(s) - 1;
+    while (p >= s && (*p == ' ' || *p == '\t'))
+	*p-- = '\0';
+    return s;
+}
+
 static int
 split(char *line, char **domain, char **ltype, char **item, char **value)
 {
@@ -844,7 +863,7 @@ split(char *line, char **domain, char **ltype, char **item, char **value)
     *domain = strtok_r(line, " \t", &saveptr);
     *ltype = strtok_r(NULL, " \t", &saveptr);
     *item = strtok_r(NULL, " \t", &saveptr);
-    *value = strtok_r(NULL, "", &saveptr);
+    *value = trim(strtok_r(NULL, "", &saveptr));
 
     count = 0;
     count += set_if_null(domain, blank);
@@ -862,6 +881,7 @@ parse_config_file(pam_handle_t *pamh, const char *uname, uid_t uid, gid_t gid,
     FILE *fil;
     char *buf = NULL;
     size_t n = 0;
+    unsigned long long lineno = 0;
 
     /* check for the conf_file */
     if (ctrl & PAM_DEBUG_ARG)
@@ -885,6 +905,8 @@ parse_config_file(pam_handle_t *pamh, const char *uname, uid_t uid, gid_t gid,
         size_t j;
         uid_t min_uid = (uid_t)-1, max_uid = (uid_t)-1;
 
+        lineno++;
+
         line = buf;
         /* skip the leading white space */
         while (*line && isspace((unsigned char)*line))
@@ -1053,7 +1075,8 @@ parse_config_file(pam_handle_t *pamh, const char *uname, uid_t uid, gid_t gid,
 	    fclose(fil);
 	    return PAM_IGNORE;
         } else {
-            pam_syslog(pamh, LOG_WARNING, "invalid line '%s' - skipped", line);
+            pam_syslog(pamh, LOG_WARNING, "invalid line %llu in '%s' - skipped",
+		       lineno, pl->conf_file);
 	}
     }
     free(buf);
@@ -1232,6 +1255,7 @@ pam_sm_open_session (pam_handle_t *pamh, int flags UNUSED,
     int ctrl;
     struct pam_limit_s plstruct;
     struct pam_limit_s *pl = &plstruct;
+    char *free_filename = NULL;
 
     D(("called."));
 
@@ -1292,13 +1316,18 @@ pam_sm_open_session (pam_handle_t *pamh, int flags UNUSED,
             if (retval != PAM_SUCCESS)
                 break;
         }
-        for (i = 0; filename_list[i] != NULL; i++)
-            free(filename_list[i]);
+        for (i = 0; filename_list[i] != NULL; i++) {
+	    if (filename_list[i] == pl->conf_file)
+		free_filename = filename_list[i];
+	    else
+		free(filename_list[i]);
+	}
         free(filename_list);
     }
 
     if (retval == PAM_IGNORE) {
         D(("the configuration file ('%s') has an applicable '<domain> -' entry", pl->conf_file));
+        free(free_filename);
         free(pl->login_group);
         return PAM_SUCCESS;
     }
@@ -1307,11 +1336,13 @@ out:
     if (retval != PAM_SUCCESS)
     {
 	pam_syslog(pamh, LOG_ERR, "error parsing the configuration file: '%s' ", pl->conf_file);
+	free(free_filename);
 	free(pl->login_group);
 	return retval;
     }
 
     retval = setup_limits(pamh, pwd->pw_name, pwd->pw_uid, ctrl, pl);
+    free(free_filename);
     free(pl->login_group);
     if (retval & LOGIN_ERR)
 	pam_error(pamh, _("There were too many logins for '%s'."),
