@@ -38,9 +38,6 @@
  *
  */
 
-#define PAM_SM_AUTH
-#define PAM_SM_SESSION
-
 #include "config.h"
 
 #include <sys/stat.h>
@@ -56,45 +53,59 @@
 #include <time.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <utmp.h>
 #include <syslog.h>
 #include <paths.h>
+#ifdef WITH_OPENSSL
+#include "hmac_openssl_wrapper.h"
+#else
 #include "hmacsha1.h"
+#endif /* WITH_OPENSSL */
+
+#ifdef USE_LOGIND
+#include <systemd/sd-login.h>
+#else
+#include <utmp.h>
+#endif
 
 #include <security/pam_modules.h>
 #include <security/_pam_macros.h>
 #include <security/pam_ext.h>
 #include <security/pam_modutil.h>
+#include "pam_inline.h"
 
 /* The default timeout we use is 5 minutes, which matches the sudo default
  * for the timestamp_timeout parameter. */
 #define DEFAULT_TIMESTAMP_TIMEOUT (5 * 60)
 #define MODULE "pam_timestamp"
-#define TIMESTAMPDIR _PATH_VARRUN "/" MODULE
+#define TIMESTAMPDIR _PATH_VARRUN MODULE
 #define TIMESTAMPKEY TIMESTAMPDIR "/_pam_timestamp_key"
 
 /* Various buffers we use need to be at least as large as either PATH_MAX or
  * LINE_MAX, so choose the larger of the two. */
-#if (LINE_MAX > PATH_MAX)
+#ifndef PATH_MAX
+#define BUFLEN LINE_MAX
+#elif (LINE_MAX > PATH_MAX)
 #define BUFLEN LINE_MAX
 #else
 #define BUFLEN PATH_MAX
 #endif
 
+#define ROOT_USER 	0
+#define ROOT_GROUP 	0
+
 /* Return PAM_SUCCESS if the given directory looks "safe". */
 static int
 check_dir_perms(pam_handle_t *pamh, const char *tdir)
 {
-	char scratch[BUFLEN];
+	char scratch[BUFLEN] = {};
 	struct stat st;
-	int i;
+	size_t i;
 	/* Check that the directory is "safe". */
 	if ((tdir == NULL) || (strlen(tdir) == 0)) {
 		return PAM_AUTH_ERR;
 	}
 	/* Iterate over the path, checking intermediate directories. */
-	memset(scratch, 0, sizeof(scratch));
-	for (i = 0; (tdir[i] != '\0') && (i < (int)sizeof(scratch)); i++) {
+	for (i = 0; (i < sizeof(scratch)) && (tdir[i] != '\0'); i++) {
 		scratch[i] = tdir[i];
 		if ((scratch[i] == '/') || (tdir[i + 1] == '\0')) {
 			/* We now have the name of a directory in the path, so
@@ -151,7 +162,7 @@ check_tty(const char *tty)
 	}
 	/* Pull out the meaningful part of the tty's name. */
 	if (strchr(tty, '/') != NULL) {
-		if (strncmp(tty, "/dev/", 5) != 0) {
+		if (pam_str_skip_prefix(tty, "/dev/") == NULL) {
 			/* Make sure the device node is actually in /dev/,
 			 * noted by Michal Zalewski. */
 			return NULL;
@@ -195,10 +206,26 @@ timestamp_good(time_t then, time_t now, time_t interval)
 }
 
 static int
-check_login_time(const char *ruser, time_t timestamp)
+check_login_time(
+#ifdef USE_LOGIND
+		 uid_t uid,
+#else
+		 const char *ruser,
+#endif
+		 time_t timestamp)
 {
-	struct utmp utbuf, *ut;
 	time_t oldest_login = 0;
+#ifdef USE_LOGIND
+#define USEC_PER_SEC  ((uint64_t) 1000000ULL)
+	uint64_t usec = 0;
+
+	if (sd_uid_get_login_time(uid, &usec) < 0) {
+	        return PAM_SERVICE_ERR;
+	}
+
+	oldest_login = usec/USEC_PER_SEC;
+#else
+	struct utmp utbuf, *ut;
 
 	setutent();
 	while(
@@ -219,6 +246,7 @@ check_login_time(const char *ruser, time_t timestamp)
 		}
 	}
 	endutent();
+#endif
 	if(oldest_login == 0 || timestamp < oldest_login) {
 		return PAM_AUTH_ERR;
 	}
@@ -282,8 +310,10 @@ get_timestamp_name(pam_handle_t *pamh, int argc, const char **argv,
 		}
 	}
 	for (i = 0; i < argc; i++) {
-		if (strncmp(argv[i], "timestampdir=", 13) == 0) {
-			tdir = argv[i] + 13;
+		const char *str;
+
+		if ((str = pam_str_skip_prefix(argv[i], "timestampdir=")) != NULL) {
+			tdir = str;
 			if (debug) {
 				pam_syslog(pamh, LOG_DEBUG,
 				       "storing timestamps in `%s'",
@@ -296,10 +326,7 @@ get_timestamp_name(pam_handle_t *pamh, int argc, const char **argv,
 		return i;
 	}
 	/* Get the name of the target user. */
-	if (pam_get_user(pamh, &user, NULL) != PAM_SUCCESS) {
-		user = NULL;
-	}
-	if ((user == NULL) || (strlen(user) == 0)) {
+	if (pam_get_user(pamh, &user, NULL) != PAM_SUCCESS || user[0] == '\0') {
 		return PAM_AUTH_ERR;
 	}
 	if (debug) {
@@ -354,7 +381,8 @@ get_timestamp_name(pam_handle_t *pamh, int argc, const char **argv,
 static void
 verbose_success(pam_handle_t *pamh, long diff)
 {
-	pam_info(pamh, _("Access granted (last access was %ld seconds ago)."), diff);
+	pam_info(pamh, _("Access has been granted"
+			 " (last access was %ld seconds ago)."), diff);
 }
 
 int
@@ -376,8 +404,10 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 		}
 	}
 	for (i = 0; i < argc; i++) {
-		if (strncmp(argv[i], "timestamp_timeout=", 18) == 0) {
-			tmp = strtol(argv[i] + 18, &p, 0);
+		const char *str;
+
+		if ((str = pam_str_skip_prefix(argv[i], "timestamp_timeout=")) != NULL) {
+			tmp = strtol(str, &p, 0);
 			if ((p != NULL) && (*p == '\0')) {
 				interval = tmp;
 				if (debug) {
@@ -449,6 +479,13 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 			return PAM_AUTH_ERR;
 		}
 
+#ifdef WITH_OPENSSL
+		if (hmac_size(pamh, debug, &maclen)) {
+			return PAM_AUTH_ERR;
+		}
+#else
+		maclen = hmac_sha1_size();
+#endif /* WITH_OPENSSL */
 		/* Check that the file is the expected size. */
 		if (st.st_size == 0) {
 			/* Invalid, but may have been created by sudo. */
@@ -456,7 +493,7 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 			return PAM_AUTH_ERR;
 		}
 		if (st.st_size !=
-		    (off_t)(strlen(path) + 1 + sizeof(then) + hmac_sha1_size())) {
+		    (off_t)(strlen(path) + 1 + sizeof(then) + maclen)) {
 			pam_syslog(pamh, LOG_NOTICE, "timestamp file `%s' "
 			       "appears to be corrupted", path);
 			close(fd);
@@ -487,8 +524,17 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 		message_end = message + strlen(path) + 1 + sizeof(then);
 
 		/* Regenerate the MAC. */
-		hmac_sha1_generate_file(pamh, &mac, &maclen, TIMESTAMPKEY, 0, 0,
-					message, message_end - message);
+#ifdef WITH_OPENSSL
+		if (hmac_generate(pamh, debug, &mac, &maclen, TIMESTAMPKEY,
+				  ROOT_USER, ROOT_GROUP, message, message_end - message)) {
+			close(fd);
+			free(message);
+			return PAM_AUTH_ERR;
+		}
+#else
+		hmac_sha1_generate_file(pamh, &mac, &maclen, TIMESTAMPKEY,
+					ROOT_USER, ROOT_GROUP, message, message_end - message);
+#endif /* WITH_OPENSSL */
 		if ((mac == NULL) ||
 		    (memcmp(path, message, strlen(path)) != 0) ||
 		    (memcmp(mac, message_end, maclen) != 0)) {
@@ -509,7 +555,15 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
 			close(fd);
 			return PAM_AUTH_ERR;
 		}
+#ifdef USE_LOGIND
+		struct passwd *pwd = pam_modutil_getpwnam(pamh, ruser);
+		if (pwd == NULL) {
+		  return PAM_SERVICE_ERR;
+		}
+		if (check_login_time(pwd->pw_uid, then) != PAM_SUCCESS)
+#else
 		if (check_login_time(ruser, then) != PAM_SUCCESS)
+#endif
 		{
 			pam_syslog(pamh, LOG_NOTICE, "timestamp file `%s' is "
 			       "older than oldest login, disallowing "
@@ -577,10 +631,10 @@ pam_sm_open_session(pam_handle_t *pamh, int flags UNUSED, int argc, const char *
 
 	/* Create the directory for the timestamp file if it doesn't already
 	 * exist. */
-	for (i = 1; path[i] != '\0'; i++) {
+	for (i = 1; i < (int) sizeof(path) && path[i] != '\0'; i++) {
 		if (path[i] == '/') {
 			/* Attempt to create the directory. */
-			strncpy(subdir, path, i);
+			memcpy(subdir, path, i);
 			subdir[i] = '\0';
 			if (mkdir(subdir, 0700) == 0) {
 				/* Attempt to set the owner to the superuser. */
@@ -605,8 +659,16 @@ pam_sm_open_session(pam_handle_t *pamh, int flags UNUSED, int argc, const char *
 		}
 	}
 
+#ifdef WITH_OPENSSL
+	if (hmac_size(pamh, debug, &maclen)) {
+		return PAM_SESSION_ERR;
+	}
+#else
+	maclen = hmac_sha1_size();
+#endif /* WITH_OPENSSL */
+
 	/* Generate the message. */
-	text = malloc(strlen(path) + 1 + sizeof(now) + hmac_sha1_size());
+	text = malloc(strlen(path) + 1 + sizeof(now) + maclen);
 	if (text == NULL) {
 		pam_syslog(pamh, LOG_CRIT, "unable to allocate memory: %m");
 		return PAM_SESSION_ERR;
@@ -621,15 +683,21 @@ pam_sm_open_session(pam_handle_t *pamh, int flags UNUSED, int argc, const char *
 	p += sizeof(now);
 
 	/* Generate the MAC and append it to the plaintext. */
-	hmac_sha1_generate_file(pamh, &mac, &maclen,
-				TIMESTAMPKEY,
-				0, 0,
-				text, p - text);
+#ifdef WITH_OPENSSL
+	if (hmac_generate(pamh, debug, &mac, &maclen, TIMESTAMPKEY,
+			  ROOT_USER, ROOT_GROUP, text, p - text)) {
+		free(text);
+		return PAM_SESSION_ERR;
+	}
+#else
+	hmac_sha1_generate_file(pamh, &mac, &maclen, TIMESTAMPKEY,
+				ROOT_USER, ROOT_GROUP, text, p - text);
 	if (mac == NULL) {
 		pam_syslog(pamh, LOG_ERR, "failure generating MAC: %m");
 		free(text);
 		return PAM_SESSION_ERR;
 	}
+#endif /* WITH_OPENSSL */
 	memmove(p, mac, maclen);
 	p += maclen;
 	free(mac);
@@ -684,13 +752,16 @@ pam_sm_close_session(pam_handle_t *pamh UNUSED, int flags UNUSED, int argc UNUSE
 int
 main(int argc, char **argv)
 {
-	int i, retval = 0, dflag = 0, kflag = 0;
+	int i, retval, dflag = 0, kflag = 0;
 	const char *target_user = NULL, *user = NULL, *tty = NULL;
 	struct passwd *pwd;
 	struct timeval tv;
 	fd_set write_fds;
 	char path[BUFLEN];
 	struct stat st;
+#ifdef USE_LOGIND
+	uid_t uid;
+#endif
 
 	/* Check that there's nothing funny going on with stdio. */
 	if ((fstat(STDIN_FILENO, &st) == -1) ||
@@ -726,7 +797,7 @@ main(int argc, char **argv)
 	if (geteuid() != 0) {
 		fprintf(stderr, "%s must be setuid root\n",
 			argv[0]);
-		retval = 2;
+		return 2;
 	}
 
 	/* Check that we have a controlling tty. */
@@ -744,49 +815,52 @@ main(int argc, char **argv)
 	/* Get the name of the invoking (requesting) user. */
 	pwd = getpwuid(getuid());
 	if (pwd == NULL) {
-		retval = 4;
+		fprintf(stderr, "unknown user\n");
+		return 4;
 	}
+#ifdef USE_LOGIND
+	uid = pwd->pw_uid;
+#endif
 
 	/* Get the name of the target user. */
 	user = strdup(pwd->pw_name);
 	if (user == NULL) {
-		retval = 4;
-	} else {
-		target_user = (optind < argc) ? argv[optind] : user;
-		if ((strchr(target_user, '.') != NULL) ||
-		    (strchr(target_user, '/') != NULL) ||
-		    (strchr(target_user, '%') != NULL)) {
-			fprintf(stderr, "unknown user: %s\n",
-				target_user);
-			retval = 4;
-		}
+		fprintf(stderr, "out of memory\n");
+		return 4;
+	}
+	target_user = (optind < argc) ? argv[optind] : user;
+	if ((strchr(target_user, '.') != NULL) ||
+	    (strchr(target_user, '/') != NULL) ||
+	    (strchr(target_user, '%') != NULL)) {
+		fprintf(stderr, "invalid user: %s\n", target_user);
+		return 4;
 	}
 
 	/* Sanity check the tty to make sure we should be checking
 	 * for timestamps which pertain to it. */
-	if (retval == 0) {
-		tty = check_tty(tty);
-		if (tty == NULL) {
-			fprintf(stderr, "invalid tty\n");
-			retval = 6;
-		}
+	tty = check_tty(tty);
+	if (tty == NULL) {
+		fprintf(stderr, "invalid tty\n");
+		return 6;
+	}
+
+	/* Generate the name of the timestamp file. */
+	if (format_timestamp_name(path, sizeof(path), TIMESTAMPDIR,
+				  tty, user, target_user) >= (int) sizeof(path)) {
+		fprintf(stderr, "path too long\n");
+		return 4;
 	}
 
 	do {
-		/* Sanity check the timestamp directory itself. */
-		if (retval == 0) {
+		retval = 0;
+		do {
+			/* Sanity check the timestamp directory itself. */
 			if (check_dir_perms(NULL, TIMESTAMPDIR) != PAM_SUCCESS) {
 				retval = 5;
+				break;
 			}
-		}
 
-		if (retval == 0) {
-			/* Generate the name of the timestamp file. */
-			format_timestamp_name(path, sizeof(path), TIMESTAMPDIR,
-					      tty, user, target_user);
-		}
 
-		if (retval == 0) {
 			if (kflag) {
 				/* Remove the timestamp. */
 				if (lstat(path, &st) != -1) {
@@ -796,17 +870,21 @@ main(int argc, char **argv)
 				/* Check the timestamp. */
 				if (lstat(path, &st) != -1) {
 					/* Check oldest login against timestamp */
+#ifdef USE_LOGIND
+					if (check_login_time(uid, st.st_mtime) != PAM_SUCCESS) {
+#else
 					if (check_login_time(user, st.st_mtime) != PAM_SUCCESS) {
+#endif
 						retval = 7;
-					} else if (!timestamp_good(st.st_mtime, time(NULL),
-							    DEFAULT_TIMESTAMP_TIMEOUT) == PAM_SUCCESS) {
+					} else if (timestamp_good(st.st_mtime, time(NULL),
+							DEFAULT_TIMESTAMP_TIMEOUT) != PAM_SUCCESS) {
 						retval = 7;
 					}
 				} else {
 					retval = 7;
 				}
 			}
-		}
+		} while (0);
 
 		if (dflag > 0) {
 			struct timeval now;
@@ -826,7 +904,6 @@ main(int argc, char **argv)
 			select(STDOUT_FILENO + 1,
 			       NULL, NULL, &write_fds,
 			       &tv);
-			retval = 0;
 		}
 	} while (dflag > 0);
 
